function [tau, acf] = calc_acf(interval, bin, fitornot)

% [t, a] = calc_acf(interval, bin)
%
% Takes a dataset of duration files, and computes the autocorrelation
% function (ACF).  
% INPUT - interval: length of the ACF that will be calculated (s)
%         bin: bin size when calculating the ACF (s)
%         fit: if fit = 1, fit the data
%              if fit = 0, don't fit the data
% OUTPUTS - tau - tau values of the ACF (s)
%           acf - a(i) is the value of the ACF at tau(i) 

interval = interval*10^9;
bin = bin*10^9;

t = [];
% Select the data to analyze, and input how many files total to analyze
cd '\Documents and Settings\David Altman\Desktop\'
[filename,direc] = uigetfile('*.*','Choose your first file');
cd(direc)
numfiles = input('How many files?');
fname = filename(1:length(filename)-1);

% For each file, the program creates a vector containing the durations
% between all events - this stops when the length of the durations are 
% longer than "interval"

totalfilelength = [];
meanlengths = [];
for(i=0:numfiles-1)
    [durations] = openFCSfile(fname, i);
    N = length(durations);
    meanlengths = [meanlengths, mean(durations)];
    totalfilelength = [totalfilelength, sum(durations)];
    alldurations = calcdur(durations);
    autocorrlist = alldurations(find(alldurations<interval));
    filelength = autocorrlist(end);
    t = [t; autocorrlist];
end
% Bin the data using the provided bin size "bin"
% Normalize according to the bins

acf = [];
tau = [];
Nsub = 50;
subbin = bin/(10^9*Nsub);   
for i = 0:1:Nsub
    tausub = [0+i*subbin:bin:interval+i*subbin];
    [acfsub] = hist(t, tausub);
    Nbins = round(filelength / bin);
    Nbinstotal = (mean(totalfilelength)) / bin;
    normalization = [Nbinstotal+i*subbin:-1:Nbinstotal-length(acfsub)+1+i*subbin];
    acfnorm = acfsub ./ normalization;
    acf = [acf, acfnorm];
    tau = [tau, tausub];
end

tau = tau/10^9;  % convert to seconds
tau = tau*10^3;   % convert to ms
%tau = tau(1:length(tau)-1);
%acf = acf(1:length(acf)-1);

disp(['Mean file length: ', num2str(mean(totalfilelength)/10^6), ' +/- ', num2str(std(totalfilelength)/10^6), ' (ms)']);
disp(['Mean duration: ', num2str(mean(meanlengths)/10^6),' +/- ', num2str(std(meanlengths)/10^6), ' (ms)']);
figure(2)
plot([0:1:length(meanlengths)-1], totalfilelength/10^6, 'r.', 'MarkerSize', 30)
xlabel('File #')
ylabel('mean file length (ms)', 'FontSize', 14);
set(gca, 'FontSize', 13');
set(gcf, 'Color', 'w');
box off

% fit to function ACF = A * ( 1 + tau/td )^-1 * (1 + B * tau/td)^(-1/2) )
% fit variables: A, B, td, C

figure(1)
plot(tau, acf,'b.', 'MarkerSize', 16)
axis([0, max(tau), min(acf), max(acf)])
%set(gca, 'FontSize', 13' );
set(gcf, 'Color', 'w');
ylabel('ACF', 'FontSize', 14);
xlabel('tau (ms)', 'FontSize', 14);


if fitornot == 1
    % Initial guesses
    Ao = max(acf);
    tdo = 0.1;          % ms
    r = 0.25*10^(-6);
    z = 1*10^(-6);
    w = r/z;
    Bo = w^2;
    Co = 1;
    s = fitoptions('Method','NonlinearLeastSquares', 'Upper', [Bo*1000, tdo*1000, 10^9], 'Lower', [0, 0, 0],  'Startpoint',[Bo, tdo, Co]);
    ACF = fittype('A * (( 1 + x/td ).^(-1)) .* ((1 + B * x/td).^(-1/2))+C', 'problem', 'A', 'options', s);
    [acffit,gof2] = fit(tau', acfnorm', ACF,  'problem', Ao );
    hold on
    plot(acffit, 'g')
    hold off
    acffit
    set(findobj('Type','line'),'LineWidth',5);
end

box off

fclose('all');
%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%  SUBROUTINES %%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%

% Opens and reads duration from datafiles
function [durations] = openFCSfile(fname, i);
h = fopen(sprintf('%s%g',fname,i));
durationfile = fscanf(h, '%c');
durations = strread(durationfile);
durations = durations;
eventtimetrace = cumsum(durations);
%disp(['Mean Duration: ', num2str(mean(durations)/(10^6)), ' (ms)']);
disp(['File #: ', num2str(i)]);
fclose(h);

% this function creates a list of durations between all the events
function [alldurations] = calcdur(durations);
N = length(durations);
dmatrix = [];
durations = durations';
dmatrix = gallery('triw', N, 1, N)*diag(durations);
imatrix = rot90(gallery('triw', N, 1, N));
alldurations = dmatrix*imatrix;
alldurations = alldurations(find(alldurations>0));
